/*
	Copyright (C) 2005 dxp

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "globals.h"

/* ----------------------------------- Error () ---------------------------- */
void
Error (const int quit, const int errcode, const char *msg)
{
	fprintf (stderr, "%i\t%s\n", errcode, msg);

	if (quit)
		exit (errcode);
}
/* ----------------------------------- Usage () ---------------------------- */
void
Usage (const uint32_t found)
{
	unsigned long int	sec, micro;
	int			rv;
	

	rv = getrusage (RUSAGE_SELF, &usage);
	if (rv == -1)
		Error (1, errno, "getrusage()");

	sec = usage.ru_utime.tv_sec + usage.ru_stime.tv_sec;
	micro = usage.ru_utime.tv_usec + usage.ru_stime.tv_usec;
	
	if (micro > 1000000)
		{
			sec++;
			micro -= 1000000;
		}
	
	if (found)
		printf ("Total Time: %li.%03lis\n", sec, micro / 1000);
	else
		printf (" %li.%03lis\n", sec, micro / 1000);
}
/* ----------------------------------- found () ---------------------------- */
void
found (const char *word, const char *chash, const char *ihash, const int mode)
{
	signal (SIGIO, SIG_IGN);
	
	printf ("\n***CRACKED ");
	switch (mode)
		{
			case 1:	printf ("MD5");	break;
			case 2:	printf ("SHA1");	break;
			default:	printf ("Unknown");	break;
		}
	printf (" HASH***\n");

	printf ("String:        \t%s\n", word);
	printf ("Computed HASH: \t%s\n", chash);
	printf ("Input HASH:    \t%s\n", ihash);
	Usage (1);
	
	printf("Words tried: %.0f\nTotal characters: %.0f\n", (double) cnt_words, (double) cnt_chars);
	print_stats ();
			
	Exit (EXIT_SUCCESS);
}
/* ----------------------------------- hash2digest () ---------------------- */
unsigned char *
hash2digest (const char *hash)
{
	static unsigned char	digest[21];
	unsigned int		len = strlen (hash); 
	unsigned int		i, di = 0;
	unsigned char		x;

	for (i = 0; i < len; i += 2) {
		x = 16 * hex2int (hash[i]);
		x = x + hex2int (hash[i + 1]);
		digest[di++] = x;
	}
	digest[di] = '\0';
	pd_len = di - 1;
	
	return digest;
}
/* ----------------------------------- hex2int () ---------------------- */
unsigned char
hex2int (const char hex)
{
	static unsigned char	binary;
	unsigned char		uppercase;

	uppercase = toupper (hex);
	if ( uppercase > 47 && uppercase < 58)
		binary = uppercase - 48;
	else
		binary = uppercase - 55;

	return binary;
}	
/* ----------------------------------- digestcmp () ---------------------- */
uint32_t
digestcmp (const unsigned char *digest)
{
	unsigned int	i;

	// Compare given digest against the computed, check in blocks of 4 bytes.
	for (i = 0; i < pd_len; i += 4)
		if ( *(unsigned int *) (password_digest + i) != *(unsigned int *) (digest + i) )
			return 1;

	return 0;
}
/* ----------------------------------- digest2hash () ---------------------- */
char *
digest2hash (const unsigned char *digest, const int mode)
{
	int	j;
	static char		output[41];

/*
	for (i = 0; i < x; i++)
		{
			j = digest[i];
			output[i << 1] = fasthex[ (j >> 4) & 0x0f ];
			output[ (i << 1) + 1 ] = fasthex[j & 0x0f];
		}
	
	output[x << 1] = 0;
*/
/*
	We'll use a macro to convert with constant index,
	which will be faster then variable incrementation.
*/
	switch (mode)
		{
			case 1:
				convert (0);
				convert (1);
				convert (2);
				convert (3);
				convert (4);
				convert (5);
				convert (6);
				convert (7);
				convert (8);
				convert (9);
				convert (10);
				convert (11);
				convert (12);
				convert (13);
				convert (14);
				convert (15);
				output[32] = 0;
				break;

			case 2:
				convert (0);
				convert (1);
				convert (2);
				convert (3);
				convert (4);
				convert (5);
				convert (6);
				convert (7);
				convert (8);
				convert (9);
				convert (10);
				convert (11);
				convert (12);
				convert (13);
				convert (14);
				convert (15);
				convert (16);
				convert (17);
				convert (18);
				convert (19);
				output[40] = 0;
				break;
		}

	return output;
}

/* ----------------------------------- brute2 () ---------------------- */
/*
	Main loop which calls the recursive function while advancing
	characters in the word arrary.
*/
void
brute2 (void)
{
	uint32_t	pos, x;

	x = wlen;
	memset(word_rf, '\0', MAX_WORD_SIZE);		
	for (pos = 0; pos < wlen; pos++)
		word_rf[pos] = alphabet[0];
	// launch threads
	for (pos = wlen; pos < MAX_WORD_SIZE; pos++)	{
		if (pos < wlen)
			printf("\t\t Total Time after Length of %i: ", x++);
		else
			printf ("\t\t Total Time after Length of %i: ", pos);
		
		rf (pos);
		wlen++;
		Usage (0);
	}	
}
/* ----------------------------------- rf () ---------------------- */
/*
	Recursive function which calls the calculation() for every
	character position within a word or itself when every
	character in a position has been tried and we need to advance.
*/
void
rf (const int pos)
{
	uint32_t	i, j = 0, alphabet_len;
	
	alphabet_len = strlen(alphabet);
	
	if (pos == 1) {
		for (i = wlen - 1; j < alphabet_len; j++) {
			// if I locked then adjust word and clear my local lock, otherwise Lock
			word_rf[i] = alphabet[j];
			cnt_words++;
			word_rf_len = strlen((char *)word_rf);
			// Unlock
			cnt_chars += word_rf_len;
			calculation(); // brute force
		}
	}
	else {
		for (i = wlen - pos; j < alphabet_len; j++) {
			// Lock
			word_rf[i] = alphabet[j];
			rf(pos - 1);
		}
	}	
}
/* ----------------------------------- signal_handler () ---------------------- */
/*
 * SIGIO handler
 */
void
signal_handler (int sig)
{
	int	c = 0, bytes = 0;

	bytes = read(STDIN_FILENO, &c, 1);
	if (bytes > 0) {
		print_stats ();
#ifdef DEBUG
	if (mode == 0)
		printf ("single\n");
#endif
	}

}
/* ----------------------------------- reset_input_mode () ---------------------- */
void
reset_input_mode (void)
{
	int	x, flags;

	// Drain the queues
	tcdrain (STDIN_FILENO);
	tcdrain (STDOUT_FILENO);	
	
	// Restore previous terminal attributes	
	x = tcsetattr (STDIN_FILENO, TCSANOW, &saved_attr);
	if (x)
		Error (0, 0, "Error restoring terminal");
	
	// Restore previous descriptor flags
	flags = fcntl (STDIN_FILENO, F_GETFL);
	if (flags != -1) {
		flags |= ~(O_ASYNC);
		fcntl (STDIN_FILENO, F_SETFL, flags);
	}
}
/* ----------------------------------- print_stats () ---------------------- */
/*
 * Display various runtime statistics.
 */
void
print_stats (void)
{
	struct rusage	usg;
	long int		sec, micro;
	int				rv;
	
	// Populate the usage structure
	rv = getrusage (RUSAGE_SELF, &usg);
	if (rv == -1)
		Error (1, errno, "print_stats: getrusage()");

	// Calculate elapsed amount of seconds
	sec = usg.ru_utime.tv_sec + usg.ru_stime.tv_sec;		// combine user and os time (seconds)
	micro = usg.ru_utime.tv_usec + usg.ru_stime.tv_usec;	// combine user and os time (micro seconds)
	if (micro > 1000000)	// adjust seconds if there's enough micro seconds
		sec++;

#ifdef DEBUG
	printf ("\n---> ! %s : print_stats() : printf\n", __FILE__);
#endif
	printf ("Time: %li sec ; Words: %.0f (%.0u K/s) ;  Characters: %.0f (%.0u K/s)\n", sec, \
		(double) cnt_words, (unsigned int) (cnt_words / sec) / 1024, (double) cnt_chars, (unsigned int) (cnt_chars / sec) / 1024);
}
void
Exit (const int status)
{
	reset_input_mode ();
	exit (status);
}
/* ----------------------------------- get_index () ---------------------- */
/*
unsigned int
get_index (const char ch)
{
	unsigned int	i = 0;

	while (1) {
		if(ch == alphabet[i++])
			break;
	}
	return i - 1;
}
*/
