/*
	Copyright (C) 2005 dxp

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "functions.h"
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/resource.h>
#include <inttypes.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>

#define MAX_WORD_SIZE 32

uint32_t		dig_off;	// Offset within a digest for comparisons.
uint32_t		err;	// Return value after comparing digests.
uint32_t		word_rf_len;	// Length of generated ascii word to brute.
uint32_t		pd_len = 0;	// Length of password_digest.
uint32_t		wlen = 1;
uint64_t		cnt_words = 0;	// passwords
uint64_t		cnt_chars = 0;	// characters
uint8_t			word_rf[128];	// Generated password.
unsigned char	*password_digest;	// Binary version of unknown password's hash.
uint8_t			sha1_digest[21];
uint8_t			md5_digest[16];

const char   	fasthex[] = "0123456789abcdef\0";
char			password_hash[128];	// Hash of the unknown password, user supplied.
struct rusage	usage;
struct termios	saved_attr, term;
FILE			*fd;
uint32_t		mode;	// Operating modes (0 = single, 1 = file).

/*
 * For md5/md5c_opt.c reorder variable is needed for optimizations.
 * As specified from the modified md5.h by J. Touch (rfc-1810),
 * for Little-Endiands it should be 0 (zero) and for Big-Endians 1 (one).
 */
#ifdef __sparc__
	int	reorder = 1;
#else
	int	reorder = 0;
#endif

/* Alphabet sets, the lower the set number the faster result will be computed. */
char				alphabet[128];
const char			set_1[] = "0123456789\0";
const char			set_2[] = "0123456789abcdef\0";
const char			set_3[] = "0123456789ABCDEF\0";
const char			set_4[] = "abcdefghijklmnopqrstuvwxyz\0";
const char			set_5[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ\0";
const char			set_6[] = "abcdefghijklmnopqrstuvwxyz0123456789\0";
const char			set_7[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789\0";
const char			set_8[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\0";
//const char			set_9[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789\0";
/* English language frequency analysis */
const char			set_9[] = "etaoinshrdlcumwfgypbvkjxqzETAOINSHRDLCUMWFGYPBVKJXQZ0123456789\0";
char				set_10[128];
/*****************************************************************************/
int
main (int argc, char *argv[])
{
	int				min, set, flags;
	uint16_t		x, i = 0;

	if (argc != 4)
		{
			printf (VERSION);
			printf ("Usage: unhash SIZE SET HASH\n");
			printf ("Performs a brute-force attack on the given HASH.\n");
			printf ("Only MD5 and SHA1 hashes!\n");
			printf ("\n");
			printf ("SIZE\tMinimum amount of letters to start with.\n");
			printf ("\n");
			printf ("SET \tNumber of the alphabet set to acquire letters from.\n");
			printf ("    \t1. Numeric only\n");
			printf ("    \t2. Hexadecimal in lowercase\n");
			printf ("    \t3. HEXadecimal in uppercase\n");
			printf ("    \t4. alpha only in lowercase\n");
			printf ("    \t5. ALPHA only in uppercase\n");
			printf ("    \t6. alphanumeric in lowercase\n");
			printf ("    \t7. ALPHANUMERIC in uppercase\n");
			printf ("    \t8. All alpha only, both uppercase and lowercase\n");
			printf ("    \t9. Alpha-Numeric, both uppercase and lowercase\n");
			printf ("    \t10. All printable ascii characters\n");
			printf ("\n");
			printf ("HASH\tHashed string to attack.\n");

			exit (EXIT_SUCCESS);
		}

	memset (alphabet, '\0', sizeof (alphabet) );
	memset (password_hash, '\0', sizeof (password_hash) );

	// Read arguments
	sscanf (argv[1], "%i", &min);
	sscanf (argv[2], "%i", &set);
	fd = fopen (argv[3], "r");
	// single crack mode if 3rd argument is not a file
	if (!fd) {
		mode = 0;
		snprintf (password_hash, sizeof (password_hash), "%s", argv[3]);
		password_digest = hash2digest(password_hash);
	}
	else
		mode = 1;

	switch ( strlen (password_hash) )
		{
			case 32:	calculation = md5crack;
						printf ("Computing MD5\n");
						break;
			case 40:	calculation = sha1crack;
						printf ("Computing SHA1\n");
						break;
			default:	Error (1, 0, "Incorrect HASH");	break;
		}
	
	switch (set)
		{
			case 1:		strncpy (alphabet, set_1, strlen (set_1) );	break;
			case 2:		strncpy (alphabet, set_2, strlen (set_2) );	break;
			case 3:		strncpy (alphabet, set_3, strlen (set_3) );	break;
			case 4:		strncpy (alphabet, set_4, strlen (set_4) );	break;
			case 5:		strncpy (alphabet, set_5, strlen (set_5) );	break;
			case 6:		strncpy (alphabet, set_6, strlen (set_6) );	break;
			case 7:		strncpy (alphabet, set_7, strlen (set_7) );	break;
			case 8:		strncpy (alphabet, set_8, strlen (set_8) );	break;
			case 9:		strncpy (alphabet, set_9, strlen (set_9) );	break;
			case 10:
						for (x = 33; x < 127; x++)
							set_10[i++] = x;
						set_10[i] = '\0';
						strncpy (alphabet, set_10, strlen (set_10) );
						break;				
			default:	Error (1, 0, "Unknow alphabet set"); break;
		}

	if (min > MAX_WORD_SIZE)
		Error(1, 0, "Number is too big");
	else {
		// Setup standard input to deliver asynchronous I/O signals.
		fcntl (STDIN_FILENO, F_SETOWN, getpid());
		flags = fcntl (STDIN_FILENO, F_GETFL);
		if (flags != -1) {
			flags |= O_ASYNC;
			fcntl (STDIN_FILENO, F_SETFL, flags);
		}		
		// Configure terminal's standard input to deliver data regardless if enter was pressed.
		if (isatty(STDIN_FILENO)) {
			tcgetattr (STDIN_FILENO, &saved_attr);
			//atexit (reset_input_mode);
			
			x = tcgetattr (STDIN_FILENO, &term);
			if (x)
				Error (1, 0, "Error obtaining terminal attributes");
			
			term.c_lflag = ~(ICANON | ECHO);
			term.c_cc[VMIN] = 0;
			term.c_cc[VTIME] = 0;
			x = tcsetattr (STDIN_FILENO, TCSANOW, &term);
			if (x)
				Error (1, 0, "Error setting terminal attributes");
			
			signal (SIGIO, signal_handler);
		}
		
		(min > 0) ? (wlen = min) : (wlen = 1);
		brute2();
	}
	
return 1;
}
